#ifndef __ABSTRACT_CONTROLLER_H__
#define __ABSTRACT_CONTROLLER_H__

#include <Ogre.h>

namespace TR
{
	/**
	 * Right handed coordinate system. Up is Y.
	 */
	class AbstractController
	{
	protected:
		/* Translations */
		double mTranslationX; // [-1 ; 1]
		double mTranslationY; // [-1 ; 1]
		double mTranslationZ; // [-1 ; 1]
		
		/* Rotations */
		double mRotationAngle; // [-inf ; +inf] but the sensible values must be between [-1 ; 1].
		double mRotationAxisX;
		double mRotationAxisY;
		double mRotationAxisZ;

		/* Actions */
		double mAction1;
		double mAction2;

	public:
		AbstractController();
		virtual ~AbstractController();
		virtual void init(Ogre::RenderWindow* window);
		virtual void release();
		virtual void prepareCapture() = 0; // Prepare the controller for the following capture.
		virtual void captureTranslation() = 0; // Sets the members to their correct values depending on the derived controller.
		virtual void captureRotation() = 0; // Sets the members to their correct values depending on the derived controller.
		virtual void captureActions() = 0; // Sets the members to their correct values depending on the derived controller.
		inline double getTranslationX() const { return mTranslationX; }
		inline double getTranslationY() const { return mTranslationY; }
		inline double getTranslationZ() const { return mTranslationZ; }
		inline double getRotationAngle() const { return mRotationAngle; }
		inline double getRotationAxisX() const { return mRotationAxisX; }
		inline double getRotationAxisY() const { return mRotationAxisY; }
		inline double getRotationAxisZ() const { return mRotationAxisZ; }
		inline double getAction1() const { return mAction1; }
		inline double getAction2() const { return mAction2; }
	};
}

#endif
