#ifndef __MENU_H__
#define __MENU_H__

#include <Ogre.h>
#include <OIS/OIS.h>

namespace TR
{
	class Menu
	{
	public:
		enum Option
		{
			OP_DEVICE_CHARACTER,
			OP_DEVICE_CAMERA,
			OP_EVT,
			OP_NEW_GAME,
			OP_QUIT
		};

		enum OptionValue
		{
			DEVICE_KEYBOARD,
			DEVICE_MOUSE,
			DEVICE_3DMOUSE,

			NEW_GAME_NO,
			NEW_GAME_TAMAX,
			NEW_GAME_RUBY,

			EVT_DEVICE_CHANGED_YES,
			EVT_DEVICE_CHANGED_NO,

			QUIT_YES,
			QUIT_NO
		};

		Menu(Ogre::ColourValue colour = Ogre::ColourValue::White,
			Ogre::ColourValue activeColour = Ogre::ColourValue::Green,
			unsigned long onKeyDownSleepingTime = 200);

		~Menu();

		inline OptionValue getOption(Option opt){ return mOptions[opt]; }
		inline bool isOpen(){ return mOpen; }
		inline void open(){ mOpen = true; mMenu1->show(); }

		void initialize(OIS::Keyboard* kb);
		void update();

	protected:
		OIS::Keyboard* mKeyboard;
		std::map<Option, OptionValue> mOptions;

		Ogre::Overlay* mMenu1;
		Ogre::Overlay* mMenu2;
		Ogre::OverlayContainer* mPanelTamax;
		Ogre::OverlayContainer* mPanelRuby;
		Ogre::OverlayElement* mNewTamax;
		Ogre::OverlayElement* mNewRuby;
		Ogre::OverlayElement* mConfig;
		Ogre::OverlayElement* mQuit;
		Ogre::OverlayElement* mDeviceCharacter;
		Ogre::OverlayElement* mDeviceCamera;
		
		Ogre::ColourValue mColour;
		Ogre::ColourValue mActiveColour;

		Ogre::Timer mTimer;
		unsigned long mOnKeyDownSleepingTime;
		bool mSleep;
		bool mOpen;

		Ogre::String mCurrentMenu; // menu_m1, menu_m2
		Ogre::String mMenu1State; // new_tamax, new_ruby, config, quit
		Ogre::String mMenu2State; // device_character, device_camera

	private:
		void moveToNextItem(Ogre::String menu);
		void moveToPreviousItem(Ogre::String menu);
		void moveToNextOption(Ogre::String item);
		void moveToPreviousOption(Ogre::String item);
	};
}

#endif

