#ifndef __OBJECT_H__
#define __OBJECT_H__

#include <Ogre.h>
#include "ObjectBody.h"

namespace TR
{
	class ObjectBody;
	class GameEngine;

	class Object
	{
	public:
		Object(GameEngine* gameEngine);
		virtual ~Object();
		inline const Ogre::Vector3& getPosition() const { return mPosition; }
		inline const Ogre::Quaternion& getOrientation() const { return mOrientation; }
		inline void setPosition(const Ogre::Vector3& position){ mPosition = position; }
		inline void setOrientation(const Ogre::Quaternion& orientation){ mOrientation = orientation; }
		inline void translate(const Ogre::Vector3& vector){ mPosition += vector; }
		inline void accelerate(const Ogre::Vector3& vector){ mLinearSpeed += vector; }
		inline ObjectBody* getObjectBody(){ return mBody; }
		virtual void updateState(const Ogre::Vector3& input, double elapsedTime); 
		inline const Ogre::Vector3& getLinearSpeed(){ return mLinearSpeed; }
		inline void setLinearSpeed(const Ogre::Vector3& linearSpeed){ mLinearSpeed = linearSpeed; }
		virtual bool isInCollision(const Object* obj);
		virtual void onCollide(Object* obj);
		inline GameEngine* getGameEngine(){ return mGameEngine; }

	protected:
		Ogre::Vector3 mPosition;
		Ogre::Quaternion mOrientation;
		Ogre::Vector3 mLinearSpeed;
		ObjectBody* mBody;
		GameEngine* mGameEngine;
	};
}

#endif
