#ifndef __RUBY_GAME_RENDERER_H__
#define __RUBY_GAME_RENDERER_H__

#include "GameRenderer.h"

namespace TR
{
	class RubyGameRenderer : public GameRenderer
	{
	public:
		enum CameraMode
		{
			CM_THIRD_PERSON_CAMERA = 0,
			CM_FIRST_PERSON_CAMERA,
			CM_OBJECT_CAMERA
		};

		enum CameraObjectAnimationType
		{
			AT_AROUND = 1,
			AT_FIXED
		};

		RubyGameRenderer(Ogre::RenderWindow* renderWindow, Ogre::Root* root, 
						 Ogre::SceneTypeMask typeMask, const Ogre::String& instanceName);
		~RubyGameRenderer();
		
		void update(double elapsedTime);
		void setCharacterBody(ObjectBody* characterBody);
		void setCameraMode(CameraMode cm);
		void setCameraObjectOptions(ObjectBody* targetBody, CameraObjectAnimationType animationType = AT_FIXED, double length = 1, Ogre::Vector3& translation = Ogre::Vector3(50, 0, -50));
		inline CameraMode getCameraMode(){ return mCameraMode; }
		inline Ogre::Camera* getCamera1(){ return mCamera1; }
		inline Ogre::Camera* getCamera3(){ return mCamera3; }
		inline Ogre::Camera* getCameraObject(){ return mCameraObj; }
		void checkCameraChange(double elapsedTime);

	private:
		Ogre::Viewport* mViewport;
		CameraMode mCameraMode;
		Ogre::Camera* mCamera1;
		Ogre::Camera* mCamera3;
		Ogre::Camera* mCameraObj;
		ObjectBody* mCameraObjTargetBody;
		double mDistanceCamera3;
		double mTotalYawCamera1;
		double mTotalPitchCamera1;
		double mCameraObjTimer;
		double mCameraObjLength;
		Ogre::Vector3 mCameraObjTranslation;
		CameraMode mCameraObjPreviousMode;
		CameraObjectAnimationType mCameraObjAnimationType;
		double mChangeCameraTimer;
		bool mCameraChangePressed;
	};
}

#endif