#ifndef __TERRAIN_SCROLLER_H__
#define __TERRAIN_SCROLLER_H__

#include <Ogre.h>

namespace TR
{
	class TerrainScroller
	{
	public:
		TerrainScroller(const Ogre::Vector2& tileDimensions = Ogre::Vector2(100,100),
						const Ogre::Vector2& nbTiles = Ogre::Vector2(2,2),
						const Ogre::Vector3& center = Ogre::Vector3(0,0,0),
						const Ogre::Vector3& depthDirection = Ogre::Vector3(0,0,1),
						const Ogre::Vector3& widthDirection = Ogre::Vector3(1,0,0),
						const Ogre::Vector2& speed = Ogre::Vector2(0.1,0.1));

		~TerrainScroller();

		inline void setTileDimensions(const Ogre::Vector2& td){ mTileDimensions = td; }
		inline void setNbTiles(const Ogre::Vector2& t){ mNbTiles = t; }
		inline void setCenter(const Ogre::Vector3& c){ mCenter = c; }
		inline void setSpeed(const Ogre::Vector2& s){ mSpeed = Ogre::Vector2(-s.x, s.y); }
		inline void setDepthDirection(const Ogre::Vector3& d){ mDepthDirection = d; }
		inline void setWidthDirection(const Ogre::Vector3& d){ mWidthDirection = d; }

		inline const Ogre::Vector3& getCenter() const { return mCenter; }
		inline double getMaximumY() { return mMaximumY; }

		inline Ogre::SceneNode* getSceneNode(){ return mSceneNode; }

		void initialize(Ogre::SceneManager* sm, const Ogre::String& name, const std::vector<std::pair<Ogre::String, double>>& mesh);//const Ogre::String& name, const Ogre::String& mesh);
		void update(double elapsedTime);
		void showBoundingBox(bool bShow);

	private:
		Ogre::Vector2 mTileDimensions;
		Ogre::Vector2 mNbTiles;
		Ogre::Vector2 mSpeed;
		Ogre::Vector3 mCenter;
		Ogre::Vector3 mDepthDirection;
		Ogre::Vector3 mWidthDirection;
		Ogre::SceneNode* mSceneNode;

		double mMaximumY;
	};
}

#endif