#include "Booster.h"
#include "BoosterBody.h"
#include "SoundManager.h"
#include "GameEngine.h"
#include "Properties.h"
#include "RubyGameRenderer.h"

namespace TR
{
	Booster::Booster(GameEngine* gameEngine): 
		EnvironmentObject(gameEngine),
		mActivated(false)
	{
		mBody = new BoosterBody(this);
		mSoundChannel = 1;
	}

	Booster::~Booster()
	{
		mGameEngine->getSoundManager()->StopSound(&mSoundChannel);
	}

	void Booster::onCollide(Object* obj)
	{
		if(!mActivated)
		{
			// Affect target.
			obj->accelerate(Ogre::Vector3(0,0,50));

			// Change camera sometimes.
			double rand = Ogre::Math::UnitRandom();
			RubyGameRenderer* gameRenderer = static_cast<RubyGameRenderer*>(mGameEngine->getGameRenderer());
			if(rand < 0.15)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_AROUND, 1.5);
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}
			else if(rand < 0.30)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_FIXED, 1.5);
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}
			else if(rand < 0.45)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_FIXED, 1.5, Ogre::Vector3(-20, -15, 35));
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}

			// Play sound.
			int sound = mGameEngine->getSoundManager()->CreateSound(Ogre::String("accel.ogg"), SOUND_TYPE_3D_SOUND);
			mGameEngine->getSoundManager()->PlaySound(sound, mBody->getSceneNode(), &mSoundChannel);
			mGameEngine->getSoundManager()->Set3DMinMaxDistance(mSoundChannel, Properties::SOUND_3D_MIN, Properties::SOUND_3D_MAX);
			
			mActivated = true;
		}
	}
}