#include "Bumper.h"
#include "BumperBody.h"
#include "GameEngine.h"
#include "Properties.h"
#include "RubyGameRenderer.h"

namespace TR
{
	Bumper::Bumper(GameEngine* gameEngine):
		EnvironmentObject(gameEngine),
		mActivated(false)
	{
		mBumperBody = new BumperBody(this);
		mBody = mBumperBody;
		mSoundChannel = 1;
	}

	Bumper::~Bumper()
	{
		mGameEngine->getSoundManager()->StopSound(&mSoundChannel);
	}

	void Bumper::onCollide(Object* obj)
	{
		if(!mActivated)
		{
			// Launch animation.
			mBumperBody->getSpringAnimationState()->setEnabled(true);
			mBumperBody->getSpringAnimationState()->setTimePosition(0);

			// Affect target.
			obj->accelerate(Ogre::Vector3(0,50,0));

			// Change camera sometimes.
			double rand = Ogre::Math::UnitRandom();
			RubyGameRenderer* gameRenderer = static_cast<RubyGameRenderer*>(mGameEngine->getGameRenderer());
			if(rand < 0.15)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_AROUND, 1.5);
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}
			else if(rand < 0.30)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_FIXED, 1.5);
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}
			else if(rand < 0.45)
			{
				gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_FIXED, 1.5, Ogre::Vector3(-20, -15, 35));
				gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);
			}
			
			// Play the sound.
			int sound = mGameEngine->getSoundManager()->CreateSound(Ogre::String("bump.ogg"), SOUND_TYPE_3D_SOUND);
			mGameEngine->getSoundManager()->PlaySound(sound, mBody->getSceneNode(), &mSoundChannel);
			mGameEngine->getSoundManager()->Set3DMinMaxDistance(mSoundChannel, Properties::SOUND_3D_MIN, Properties::SOUND_3D_MAX);
			
			mActivated = true;
		}
	}
}