#ifndef NO_3DMOUSE

#include "Com3dConnexion.h"
#include "Log.h"

namespace TR
{
	Com3dConnexion::Com3dConnexion() : 
		m3DSensor(0),
		m3DKeyboard(0),
		m3DxSimpleDevice(0)
	{

	}

	Com3dConnexion::~Com3dConnexion()
	{
		release();
	}

	bool Com3dConnexion::initialize(const char* configName)
	{
		HRESULT hr;
		CComPtr<IUnknown> _3DxDevice;

		// Create the device object
		hr = _3DxDevice.CoCreateInstance(__uuidof(Device));

		if (SUCCEEDED(hr))
		{
			hr = _3DxDevice.QueryInterface(&m3DxSimpleDevice);

			if (SUCCEEDED(hr))
			{
				// Get the interfaces to the sensor and the keyboard;
				m3DSensor = m3DxSimpleDevice->Sensor;
				m3DKeyboard = m3DxSimpleDevice->Keyboard;

				if(configName != 0)
				{
					// Associate a configuration with this device
					m3DxSimpleDevice->LoadPreferences(_T(configName));
				}

				// Connect to the driver
				m3DxSimpleDevice->Connect();

				return true;
			}
		}

		return false;
	}

	void Com3dConnexion::release()
	{
		// Release the sensor and keyboard interfaces
		if (m3DSensor)
			m3DSensor.Release();

		if (m3DKeyboard)
			m3DKeyboard.Release();

		if (m3DxSimpleDevice)
		{
			// Disconnect it from the driver
			m3DxSimpleDevice->Disconnect();
			m3DxSimpleDevice.Release();
		}
	}

	Com3dConnexion::DeviceType Com3dConnexion::getDeviceType() const 
	{
		return (DeviceType)m3DxSimpleDevice->Type;
	}

	bool Com3dConnexion::isDevicePlugged() const 
	{
		return m3DxSimpleDevice->Type != 0;
	}
}

#endif