#include "Environment.h"
#include "Booster.h"

namespace TR
{
	Environment::Environment(GameRenderer* gr, GameEngine* gameEngine):
		mRenderer(gr),
		mShowBoundingBox(false)
	{
		mGameEngine = gameEngine;
	}

	Environment::~Environment()
	{
		size_t length = mObjects.size();
		for(size_t i = 0; i < length; i++)
		{
			delete mObjects[i];
		}
	}

	void Environment::update(const Ogre::Vector3& input, double elapsedTime, Ogre::Vector2 characterSpeed, double characterAltitude)
	{
		size_t length = mObjects.size();
		for(size_t i = 0; i < length; i++)
		{
			mObjects[i]->translate(Ogre::Vector3(-characterSpeed.x * elapsedTime, 0, characterSpeed.y * elapsedTime));
			mObjects[i]->updateState(input, elapsedTime);
		}

		this->generate(elapsedTime, characterSpeed, characterAltitude);
	}

	void Environment::generate(double elapsedTime, Ogre::Vector2 characterSpeed, double characterAltitude)
	{
	}
		
	void Environment::resolveCollisions(Object* obj)
	{
		size_t length = mObjects.size();
		for(size_t i = 0; i < length; i++)
		{
			if(mObjects[i]->isInCollision(obj))
				mObjects[i]->onCollide(obj);	
		}
	}

	void Environment::showBoundingBox(bool bShow)
	{
		mShowBoundingBox = bShow;

		size_t nbObjects = mObjects.size();
		for(size_t i=0; i<nbObjects; i++)
		{
			mObjects[i]->getObjectBody()->showBoundingBox(bShow);
		}
	}
}
