#include "GameRenderer.h"

namespace TR
{
	GameRenderer::GameRenderer(Ogre::RenderWindow* renderWindow, Ogre::Root* root, Ogre::SceneTypeMask typeMask, const Ogre::String& instanceName) :
		mRenderWindow(renderWindow),
		mRoot(root),
		mCharacterBody(NULL),
		mSceneManager(NULL),
		mController(NULL),
		mEarNode(NULL)
	{
		mSceneManager = mRoot->createSceneManager(typeMask, instanceName);
	}

	GameRenderer::~GameRenderer()
	{
		mRenderWindow->removeAllViewports();
		mRoot->destroySceneManager(mSceneManager);
	}

	void GameRenderer::update(double elapsedTime)
	{
		// Update all objects to render.
		size_t size = mBodies.size();
		for(size_t i = 0; i < size; ++i)
		{
			mBodies[i]->update(elapsedTime);
		}
	}

	void GameRenderer::addToRenderQueue(ObjectBody* body)
	{
		body->buildSceneNode(mSceneManager);
		mBodies.push_back(body);
	}

	void GameRenderer::removeFromRenderQueue(ObjectBody* body)
	{
		body->destroySceneNode(mSceneManager);
		
		size_t size = mBodies.size();
		size_t index = -1;
		for(size_t i = 0; i < size && index == -1; ++i)
		{
			if(body == mBodies[i])
				index = i;
		}
		
		if(index != -1)
			mBodies.erase(mBodies.begin() + index);
	}
}
