#include "KeyboardController.h"

#include <Ogre.h>

namespace TR
{
	void KeyboardController::init(Ogre::RenderWindow* window)
	{
		OIS::ParamList pl;
		size_t windowHnd = 0;
		std::ostringstream windowHndStr;
		window->getCustomAttribute("WINDOW", &windowHnd); 
		windowHndStr << windowHnd;
		pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));
		mInputManager = OIS::InputManager::createInputSystem(pl);

		mKeyboard = static_cast<OIS::Keyboard*>(mInputManager->createInputObject(OIS::OISKeyboard, false));
	}

	void KeyboardController::release()
	{
		if(mInputManager)
		{
			mInputManager->destroyInputObject(mKeyboard);

			OIS::InputManager::destroyInputSystem(mInputManager);
			mInputManager = 0;
		}
	}

	void KeyboardController::prepareCapture()
	{
		mKeyboard->capture();
	}

	void KeyboardController::captureTranslation()
	{
		/* Translations */
		mTranslationZ = 0;
		if(mKeyboard->isKeyDown(OIS::KC_Q)) mTranslationZ += -1;
		if(mKeyboard->isKeyDown(OIS::KC_E)) mTranslationZ += 1;

		mTranslationX = 0;
		if(mKeyboard->isKeyDown(OIS::KC_A)) mTranslationX += -1;
		if(mKeyboard->isKeyDown(OIS::KC_D)) mTranslationX += 1;
		
		mTranslationY = 0;
		if(mKeyboard->isKeyDown(OIS::KC_S)) mTranslationY += 1;
		if(mKeyboard->isKeyDown(OIS::KC_W)) mTranslationY += -1;	
	}

	void KeyboardController::captureRotation()
	{
		/* Rotations */
		Ogre::Vector3 axis(0, 0, 0);
		if(mKeyboard->isKeyDown(OIS::KC_UP)) axis += Ogre::Vector3::UNIT_Y;
		if(mKeyboard->isKeyDown(OIS::KC_DOWN)) axis += Ogre::Vector3::NEGATIVE_UNIT_Y;
		if(mKeyboard->isKeyDown(OIS::KC_RIGHT)) axis += Ogre::Vector3::UNIT_X;
		if(mKeyboard->isKeyDown(OIS::KC_LEFT)) axis += Ogre::Vector3::NEGATIVE_UNIT_X;

		if(axis.isZeroLength() == false)
		{
			axis.normalise();
			axis = axis.crossProduct(Ogre::Vector3::UNIT_Z);

			mRotationAngle = 1;
			mRotationAxisX = axis.x;
			mRotationAxisY = axis.y;
			mRotationAxisZ = axis.z;
		}
		else
		{
			mRotationAngle = 0;
		}
	}

	void KeyboardController::captureActions()
	{
		mAction1 = -1;
		if(mKeyboard->isKeyDown(OIS::KC_SPACE)) mAction1 = 1;

		mAction2 = -1;
		if(mKeyboard->isKeyDown(OIS::KC_B)) mAction2 = 1;
	}
}
