#include "Menu.h"

using namespace Ogre;
using namespace OIS;

namespace TR
{
	Menu::Menu(ColourValue colour, ColourValue activeColour, unsigned long onKeyDownSleepingTime)
	{
		mColour = colour;
		mActiveColour = activeColour;
		mOnKeyDownSleepingTime = onKeyDownSleepingTime;
	}

	Menu::~Menu(){}

	void Menu::initialize(Keyboard* kb)
	{
		mKeyboard = kb;
		mSleep = false;
		mOpen = false;

		mCurrentMenu = "menu_m1";
		mMenu1State = "new_tamax";
		mMenu2State = "device_character";

		mOptions.clear();
		mOptions.insert(std::pair<Option, OptionValue>(OP_NEW_GAME, NEW_GAME_NO));
		mOptions.insert(std::pair<Option, OptionValue>(OP_QUIT, QUIT_NO));
		mOptions.insert(std::pair<Option, OptionValue>(OP_DEVICE_CHARACTER, DEVICE_KEYBOARD));
		mOptions.insert(std::pair<Option, OptionValue>(OP_DEVICE_CAMERA, DEVICE_MOUSE));
		mOptions.insert(std::pair<Option, OptionValue>(OP_EVT, EVT_DEVICE_CHANGED_NO));

		// menu 1
		mMenu1 = OverlayManager::getSingleton().getByName("Menu1");
		
		// sets panel tamax the default selection
		mPanelTamax = static_cast<OverlayContainer*>(OverlayManager::getSingleton().getOverlayElement("PanelTamax"));
		mPanelTamax->setMaterialName("TamaxHover");
		mNewTamax = OverlayManager::getSingleton().getOverlayElement("NewTamax");
		mNewTamax->setColour(mActiveColour);
		mNewTamax->hide();

		mPanelRuby = static_cast<OverlayContainer*>(OverlayManager::getSingleton().getOverlayElement("PanelRuby"));
		mNewRuby = OverlayManager::getSingleton().getOverlayElement("NewRuby");
		mNewRuby->setColour(mColour);
		mNewRuby->hide();

		mConfig = OverlayManager::getSingleton().getOverlayElement("Config");
		mConfig->setColour(mColour);

		mQuit = OverlayManager::getSingleton().getOverlayElement("Quit");
		mQuit->setColour(mColour);
		
		// menu 2
		mMenu2 = OverlayManager::getSingleton().getByName("Menu2");

		mDeviceCharacter = OverlayManager::getSingleton().getOverlayElement("CharacterDevice");
		mDeviceCharacter->setColour(mActiveColour);

		mDeviceCamera = OverlayManager::getSingleton().getOverlayElement("CameraDevice");
		mDeviceCamera->setColour(mColour);
	}

	void Menu::update()
	{
		mOptions[OP_NEW_GAME] = NEW_GAME_NO;
		mOptions[OP_QUIT] = QUIT_NO;
		mOptions[OP_EVT] = EVT_DEVICE_CHANGED_NO;

		// checks the opening
		if(!mOpen && mTimer.getMilliseconds() > mOnKeyDownSleepingTime)
		{
			if(mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			{
				mSleep = true;
				mTimer.reset();
				mOpen = true;
				mMenu1->show();
			}
			else return;
		}

		// sleeps when a key was pressed
		if(mSleep)
		{
			if(mTimer.getMilliseconds() > mOnKeyDownSleepingTime) mSleep = false;
			else return;
		}

		// menu 1
		if(mCurrentMenu == "menu_m1")
		{
			if(mKeyboard->isKeyDown(OIS::KC_DOWN))
			{
				mSleep = true;
				mTimer.reset();
				moveToNextItem(mCurrentMenu);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_UP))
			{
				mSleep = true;
				mTimer.reset();
				moveToPreviousItem(mCurrentMenu);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_RETURN))
			{
				mSleep = true;
				mTimer.reset();
				
				if(mMenu1State == "config")
				{
					// moves to the menu2
					mCurrentMenu = "menu_m2";
					mMenu1->hide();
					mMenu2->show();
				}
				else if(mMenu1State == "new_tamax")
				{
					mOptions[OP_NEW_GAME] = NEW_GAME_TAMAX;
					mSleep = true;
					mTimer.reset();
					mOpen = false;
					mMenu1->hide();
				}
				else if(mMenu1State == "new_ruby")
				{
					mOptions[OP_NEW_GAME] = NEW_GAME_RUBY;
					mSleep = true;
					mTimer.reset();
					mOpen = false;
					mMenu1->hide();
				}
				else if(mMenu1State == "quit")
				{
					mOptions[OP_QUIT] = QUIT_YES;
				}
			}
			else if(mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			{
				// escapes from the menu
				mSleep = true;
				mTimer.reset();
				mOpen = false;
				mMenu1->hide();
			}
		}
		// menu 2
		else
		{
			if(mKeyboard->isKeyDown(OIS::KC_DOWN))
			{
				mSleep = true;
				mTimer.reset();
				moveToNextItem(mCurrentMenu);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_UP))
			{
				mSleep = true;
				mTimer.reset();
				moveToPreviousItem(mCurrentMenu);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_RIGHT) || mKeyboard->isKeyDown(OIS::KC_RETURN))
			{
				mSleep = true;
				mTimer.reset();
				moveToNextOption(mMenu2State);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_LEFT))
			{
				mSleep = true;
				mTimer.reset();
				moveToPreviousOption(mMenu2State);
			}
			else if(mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			{
				// escapes from the menu2 and moves to the menu1
				mSleep = true;
				mTimer.reset();
				mCurrentMenu = "menu_m1";
				mOptions[OP_EVT] = EVT_DEVICE_CHANGED_YES;
				mMenu2->hide();
				mMenu1->show();
			}
		}
	}

	// moves to the next item in a menu
	void Menu::moveToNextItem(String menu)
	{
		if(menu == "menu_m1")
		{
			if(mMenu1State == "new_tamax")
			{
				mMenu1State = "new_ruby";
				mNewTamax->setColour(mColour);
				mPanelTamax->setMaterialName("Tamax");
				mNewRuby->setColour(mActiveColour);
				mPanelRuby->setMaterialName("RubyHover");
			}
			else if(mMenu1State == "new_ruby")
			{
				mMenu1State = "config";
				mNewRuby->setColour(mColour);
				mPanelRuby->setMaterialName("Ruby");
				mConfig->setColour(mActiveColour);
			}
			else if(mMenu1State == "config")
			{
				mMenu1State = "quit";
				mConfig->setColour(mColour);
				mQuit->setColour(mActiveColour);
			}
			else
			{
				mMenu1State = "new_tamax";
				mQuit->setColour(mColour);
				mNewTamax->setColour(mActiveColour);
				mPanelTamax->setMaterialName("TamaxHover");
			}
		}
		else
		{
			if(mMenu2State == "device_character")
			{
				mMenu2State = "device_camera";
				mDeviceCharacter->setColour(mColour);
				mDeviceCamera->setColour(mActiveColour);
			}
			else
			{
				mMenu2State = "device_character";
				mDeviceCamera->setColour(mColour);
				mDeviceCharacter->setColour(mActiveColour);
			}
		}
	}

	// moves to the previous item in a menu
	void Menu::moveToPreviousItem(String menu)
	{
		if(menu == "menu_m1")
		{
			if(mMenu1State == "quit")
			{
				mMenu1State = "config";
				mQuit->setColour(mColour);
				mConfig->setColour(mActiveColour);
			}
			else if(mMenu1State == "config")
			{
				mMenu1State = "new_ruby";
				mConfig->setColour(mColour);
				mNewRuby->setColour(mActiveColour);
				mPanelRuby->setMaterialName("RubyHover");
			}
			else if(mMenu1State == "new_ruby")
			{
				mMenu1State = "new_tamax";
				mNewRuby->setColour(mColour);
				mPanelRuby->setMaterialName("Ruby");
				mNewTamax->setColour(mActiveColour);
				mPanelTamax->setMaterialName("TamaxHover");
			}
			else
			{
				mMenu1State = "quit";
				mNewTamax->setColour(mColour);
				mQuit->setColour(mActiveColour);
				mPanelTamax->setMaterialName("Tamax");
			}
		}
		else
		{
			if(mMenu2State == "device_camera")
			{
				mMenu2State = "device_character";
				mDeviceCamera->setColour(mColour);
				mDeviceCharacter->setColour(mActiveColour);
			}
			else
			{
				mMenu2State = "device_camera";
				mDeviceCharacter->setColour(mColour);
				mDeviceCamera->setColour(mActiveColour);
			}
		}
	}

	// moves to the next option in an item
	void Menu::moveToNextOption(String item)
	{
		if(item == "device_character")
		{
			if(mOptions[OP_DEVICE_CHARACTER] == DEVICE_KEYBOARD)
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_MOUSE;
				mDeviceCharacter->setCaption("Character Device : Mouse");
			}
			else if(mOptions[OP_DEVICE_CHARACTER] == DEVICE_MOUSE)
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_3DMOUSE;
				mDeviceCharacter->setCaption("Character Device : 3D Mouse");
			}
			else
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_KEYBOARD;
				mDeviceCharacter->setCaption("Character Device : Keyboard");
			}
		}
		else
		{
			if(mOptions[OP_DEVICE_CAMERA] == DEVICE_KEYBOARD)
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_MOUSE;
				mDeviceCamera->setCaption("Camera Device : Mouse");
			}
			else if(mOptions[OP_DEVICE_CAMERA] == DEVICE_MOUSE)
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_3DMOUSE;
				mDeviceCamera->setCaption("Camera Device : 3D Mouse");
			}
			else
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_KEYBOARD;
				mDeviceCamera->setCaption("Camera Device : Keyboard");
			}
		}
	}

	// moves to the previous option in an item
	void Menu::moveToPreviousOption(String item)
	{
		if(item == "device_character")
		{
			if(mOptions[OP_DEVICE_CHARACTER] == DEVICE_3DMOUSE)
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_MOUSE;
				mDeviceCharacter->setCaption("Character Device : Mouse");
			}
			else if(mOptions[OP_DEVICE_CHARACTER] == DEVICE_MOUSE)
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_KEYBOARD;
				mDeviceCharacter->setCaption("Character Device : Keyboard");
			}
			else
			{
				mOptions[OP_DEVICE_CHARACTER] = DEVICE_3DMOUSE;
				mDeviceCharacter->setCaption("Character Device : 3D Mouse");
			}
		}
		else
		{
			if(mOptions[OP_DEVICE_CAMERA] == DEVICE_3DMOUSE)
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_MOUSE;
				mDeviceCamera->setCaption("Camera Device : Mouse");
			}
			else if(mOptions[OP_DEVICE_CAMERA] == DEVICE_MOUSE)
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_KEYBOARD;
				mDeviceCamera->setCaption("Camera Device : Keyboard");
			}
			else
			{
				mOptions[OP_DEVICE_CAMERA] = DEVICE_3DMOUSE;
				mDeviceCamera->setCaption("Camera Device : 3D Mouse");
			}
		}
	}
}