#include "MouseController.h"

#include <Ogre.h>

namespace TR
{
	void MouseController::init(Ogre::RenderWindow* window)
	{
		OIS::ParamList pl;
		size_t windowHnd = 0;
		std::ostringstream windowHndStr;
		window->getCustomAttribute("WINDOW", &windowHnd); 
		windowHndStr << windowHnd;
		pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));
		mInputManager = OIS::InputManager::createInputSystem(pl);

		mMouse = static_cast<OIS::Mouse*>(mInputManager->createInputObject(OIS::OISMouse, false));
	}

	void MouseController::release()
	{
		if(mInputManager)
		{
			mInputManager->destroyInputObject(mMouse);

			OIS::InputManager::destroyInputSystem(mInputManager);
			mInputManager = 0;
		}
	}

	void MouseController::prepareCapture()
	{
		mMouse->capture();
		mMouseState = mMouse->getMouseState();
	}

	void MouseController::captureTranslation()
	{
		/* Translations */
		if(mMouseState.buttonDown(OIS::MB_Right) == true)
		{
			Ogre::Vector2 rel(mMouseState.X.rel, -mMouseState.Y.rel);
			
			if(rel.isZeroLength() == false)
			{
				rel.normalise();
				mTranslationX = rel.x;
				mTranslationY = rel.y;
			}
		}
		else
		{
			mTranslationX = 0;
			mTranslationY = 0;
		}

		mTranslationZ = -mMouseState.Z.rel * 0.1;

		/*
		if(-mMouseState.Z.rel > 0) mTranslationZ += 1;
		else if(-mMouseState.Z.rel < 0) mTranslationZ -= 1;
		
		// Clamp values.
		if(mTranslationZ > 1.) mTranslationZ = 1.;
		if(mTranslationZ < -1.) mTranslationZ = -1.;
		*/
	}

	void MouseController::captureRotation()
	{
		/* Rotations */
		if(mMouseState.buttonDown(OIS::MB_Left) == true)
		{
			Ogre::Vector3 axis(0, 0, 0);
			axis += Ogre::Vector3::UNIT_Y * -mMouseState.Y.rel;
			axis += Ogre::Vector3::UNIT_X * mMouseState.X.rel;

			if(axis.isZeroLength() == false)
			{
				double length = axis.normalise();
				axis = axis.crossProduct(Ogre::Vector3::UNIT_Z);

				mRotationAngle = length * 3;
				mRotationAxisX = axis.x;
				mRotationAxisY = axis.y;
				mRotationAxisZ = axis.z;
			}
			else
			{
				mRotationAngle = 0;
			}
		}
		else
		{
			mRotationAngle = 0;
		}
	}

	void MouseController::captureActions()
	{

	}
}
