#include "OgreText.h"

using namespace Ogre;

namespace TR
{
	int OgreText::init = 0;

	OgreText::OgreText(const Ogre::String& fontName) :
		mFontName(fontName)
	{
		mOverlayMgr = OverlayManager::getSingletonPtr();
		if(init == 0)
		{
			mPanel = static_cast<OverlayContainer*>(mOverlayMgr->createOverlayElement("Panel","GUI"));
			mPanel->setMetricsMode(Ogre::GMM_PIXELS);
			mPanel->setPosition(10, 10);
			mPanel->setDimensions(100, 100);
			mOverlay = mOverlayMgr->create("GUI_OVERLAY");
			mOverlay->add2D(mPanel);
		}

		++(this->init);
		mElement = "element_" + StringConverter::toString(init);
		mOverlay = mOverlayMgr->getByName("GUI_OVERLAY");
		mPanel = static_cast<OverlayContainer*>(mOverlayMgr->getOverlayElement("GUI"));
		mTextArea = static_cast<TextAreaOverlayElement*>(mOverlayMgr->createOverlayElement("TextArea",mElement));
		mPanel->addChild(mTextArea);
		mOverlay->show();
	}

	OgreText::~OgreText()
	{
		mElement="element_" + StringConverter::toString(init);
		mOverlayMgr->destroyOverlayElement(mElement);
		--(this->init);
		if(init==0)
		{
			mOverlayMgr->destroyOverlayElement("GUI");
			mOverlayMgr->destroy("GUI_OVERLAY");
		}
	}

	void OgreText::setText(const char* s)
	{
		mTextArea->setCaption(s);
		mTextArea->setDimensions(5, 5);
		mTextArea->setMetricsMode(Ogre::GMM_PIXELS);
		mTextArea->setWidth(50);
		mTextArea->setHeight(50);
		mTextArea->setFontName(mFontName);
		mTextArea->setCharHeight(15);
	}

	void OgreText::setText(const Ogre::String& s)
	{
		setText(s.c_str());
	}

	void OgreText::setPos(int x, int y)
	{
		mTextArea->setPosition(x, y);
	}

	void OgreText::setColor(float R, float G, float B, float I)
	{
		mTextArea->setColour(Ogre::ColourValue(R, G, B, I));
	}
}
