#include "RubyBody.h"
#include "Log.h"

namespace TR
{
	RubyBody::RubyBody(Ruby* ruby) : 
		ObjectBody(ruby),
		mRuby(ruby),
		mCloakAnimation(NULL),
		mFlyingAnimation(NULL),
		mDustPS(NULL),
		mBloodPS(NULL),
		mDustPSEmissionRate(0),
		mBloodTimer(0),
		mCollisionEntity(NULL)
	{
	}

	RubyBody::~RubyBody(void)
	{
	}

	void RubyBody::buildSceneNode(Ogre::SceneManager* sceneMgr)
	{
		ObjectBody::buildSceneNode(sceneMgr);

		// Ruby's mesh.
		Ogre::Entity* entity = sceneMgr->createEntity("Ruby", "Ruby.mesh");
		mFlyingAnimation = entity->getAnimationState("Flying");
		mFlyingAnimation->setLoop(true);
		mFlyingAnimation->setEnabled(true);
		mSceneNode->attachObject(entity);
		mCollisionEntity = entity;

		entity = sceneMgr->createEntity("RubyCloak", "Cloak.mesh");
		mCloakAnimation = entity->getAnimationState("Cloth");
		mCloakAnimation->setLoop(true);
		mCloakAnimation->setEnabled(true);
		mSceneNode->attachObject(entity);

		mSceneNode->scale(.03, .03, .03); // Ruby's 2m high.
		mRuby->setOrientation(Ogre::Quaternion(Ogre::Degree(-70), Ogre::Vector3(1,0,0)));
	
		// Particles.
		Ogre::SceneNode* mDustPSSceneNode = sceneMgr->getRootSceneNode()->createChildSceneNode();
		mDustPSSceneNode->translate(0, 4, -1);
		mDustPS = sceneMgr->createParticleSystem("DustPS", "Dust");
		mDustPSEmissionRate = mDustPS->getEmitter(0)->getEmissionRate();
		mDustPSSceneNode->attachObject(mDustPS);

		mBloodPS = sceneMgr->createParticleSystem("BloodPS", "Blood");
	}

	void RubyBody::update(double elapsedTime)
	{
		ObjectBody::update(elapsedTime);

		mCloakAnimation->addTime(elapsedTime);
		mFlyingAnimation->addTime(elapsedTime);

		// Dust particle effect when ruby is near the floor.
		if(mRuby->isNearTerrain() && !mRuby->hasCrashed())
			mDustPS->getEmitter(0)->setEmissionRate(mDustPSEmissionRate);
		else
			mDustPS->getEmitter(0)->setEmissionRate(0);

		// Blood particle effect when ruby has crashed.
		if(mRuby->hasCrashed())
		{
			if(mBloodTimer == 0)
			{
				mSceneNode->setVisible(false);
				Ogre::SceneNode* bloodNode = mSceneNode->getParentSceneNode()->createChildSceneNode();
				bloodNode->attachObject(mBloodPS);
				bloodNode->setPosition(mSceneNode->getPosition());
			}
			else if(mBloodTimer > 1)
			{
				mBloodPS->setVisible(false);
			}
			
			mBloodTimer += elapsedTime;
		}
	}

	Ogre::AxisAlignedBox RubyBody::getCollisionAABB() const
	{
		return mCollisionEntity->getWorldBoundingBox();
	}
}
