#include "RubyEnvironment.h"
#include "Booster.h"
#include "Slower.h"
#include "Bumper.h"
#include "Stopper.h"
#include "Properties.h"

namespace TR
{
	RubyEnvironment::RubyEnvironment(GameRenderer* gr, GameEngine* ge):
			Environment(gr, ge),
			mElapsedDistance(0)
	{
		srand(time(0));
	}

	RubyEnvironment::~RubyEnvironment()
	{
	}

	void RubyEnvironment::generate(double elapsedTime, Ogre::Vector2 rubySpeed, double rubyAltitude)
	{
		if(mRenderer == NULL) return;

		mElapsedDistance += (elapsedTime * rubySpeed.y);
		
		if(mElapsedDistance > Properties::ENV_AVERAGE_DISTANCE_INTERVAL)
		{
			mElapsedDistance = 0;

			double randomObjectIndicator = Ogre::Math::UnitRandom();

			// Booster
			if(randomObjectIndicator < 0.2)
			{
				double x = Properties::ENV_WIDTH_LIMIT * Ogre::Math::SymmetricRandom();
				double y = (Properties::ENV_TOP_LIMIT - Properties::ENV_BOTTOM_LIMIT) * Ogre::Math::UnitRandom() + Properties::ENV_BOTTOM_LIMIT;
				double z = -Properties::ENV_POPING_DISTANCE + Ogre::Math::UnitRandom() * Properties::ENV_RANDOM_DISTANCE;

				Booster* object = new Booster(mGameEngine);
				mRenderer->addToRenderQueue(object->getObjectBody());
				object->setPosition(Ogre::Vector3(x,y,z));
				object->getObjectBody()->showBoundingBox(mShowBoundingBox);
				mObjects.push_back(object);

				if(Ogre::Math::UnitRandom() < 0.3)
				{
					object = new Booster(mGameEngine);
					mRenderer->addToRenderQueue(object->getObjectBody());
					object->setPosition(Ogre::Vector3(x,y,z-5));
					object->getObjectBody()->showBoundingBox(mShowBoundingBox);
					mObjects.push_back(object);
					
					if(Ogre::Math::UnitRandom() < 0.3)
					{
						object = new Booster(mGameEngine);
						mRenderer->addToRenderQueue(object->getObjectBody());
						object->setPosition(Ogre::Vector3(x,y,z-10));
						object->getObjectBody()->showBoundingBox(mShowBoundingBox);
						mObjects.push_back(object);
					}
				}
			}
			// Slower
			else if(randomObjectIndicator < 0.45)
			{
				double x = Properties::ENV_WIDTH_LIMIT * Ogre::Math::SymmetricRandom();
				double y = (Properties::ENV_TOP_LIMIT - Properties::ENV_BOTTOM_LIMIT) * Ogre::Math::UnitRandom() + Properties::ENV_BOTTOM_LIMIT;
				double z = -Properties::ENV_POPING_DISTANCE + Ogre::Math::UnitRandom() * Properties::ENV_RANDOM_DISTANCE;

				Slower* object = new Slower(mGameEngine);
				mRenderer->addToRenderQueue(object->getObjectBody());
				object->setPosition(Ogre::Vector3(x,y,z));
				object->getObjectBody()->showBoundingBox(mShowBoundingBox);
				mObjects.push_back(object);
			}
			// Bumper
			else if(randomObjectIndicator < 0.65)
			{
				double x = Properties::ENV_WIDTH_LIMIT * Ogre::Math::SymmetricRandom();
				double y = (Properties::ENV_TOP_LIMIT - Properties::ENV_BOTTOM_LIMIT) * Ogre::Math::UnitRandom() + Properties::ENV_BOTTOM_LIMIT;
				double z = -Properties::ENV_POPING_DISTANCE + Ogre::Math::UnitRandom() * Properties::ENV_RANDOM_DISTANCE;

				Bumper* object = new Bumper(mGameEngine);
				mRenderer->addToRenderQueue(object->getObjectBody());
				object->setPosition(Ogre::Vector3(x,y,z));
				object->getObjectBody()->showBoundingBox(mShowBoundingBox);
				mObjects.push_back(object);
			}
			// Obstacle
			else if(randomObjectIndicator < 0.9)
			{
				double x = Properties::ENV_WIDTH_LIMIT * Ogre::Math::SymmetricRandom();
				double y = (Properties::ENV_TOP_LIMIT - Properties::ENV_BOTTOM_LIMIT) * Ogre::Math::UnitRandom() + Properties::ENV_BOTTOM_LIMIT;
				double z = -Properties::ENV_POPING_DISTANCE + Ogre::Math::UnitRandom() * Properties::ENV_RANDOM_DISTANCE;

				Stopper* object = new Stopper(mGameEngine);
				mRenderer->addToRenderQueue(object->getObjectBody());
				object->setPosition(Ogre::Vector3(x,y,z));
				object->getObjectBody()->showBoundingBox(mShowBoundingBox);
				mObjects.push_back(object);
			}
			// else no object
		}


		// Deletes the too far objects
		size_t length = mObjects.size();
		std::vector<EnvironmentObject*>::iterator iterator = mObjects.begin();

		for(size_t i = 0; i < length; i++)
		{
			if(mObjects[i]->getPosition().squaredDistance(Ogre::Vector3(0,rubyAltitude,0)) > Properties::ENV_POPING_DISTANCE * Properties::ENV_POPING_DISTANCE * 2)
			{
				mRenderer->removeFromRenderQueue(mObjects[i]->getObjectBody());
				delete(mObjects[i]);
				mObjects.erase(iterator);
				length--;
			}

			if(iterator != mObjects.end()) iterator++;
		}
	}
}