#include "RubyHud.h"
#include "Properties.h"

namespace TR
{
	RubyHud::RubyHud():
		mSpeed(0),
		mAltitude(0),
		mDistance(0),
		mScore(0),
		mTimer(0),
		mOverlayHud(NULL),
		mElementScore(NULL),
		mElementTimeLeft(NULL),
		mElementSpeed(NULL),
		mElementAltitude(NULL),
		mElementDistance(NULL),
		mOverlayEndGame(NULL),
		mElementEndScore(NULL),
		mElementEndDistance(NULL)
	{		
		mOverlayHud = Ogre::OverlayManager::getSingleton().getByName("HUD");
		mElementScore = Ogre::OverlayManager::getSingleton().getOverlayElement("Score");
		mElementTimeLeft = Ogre::OverlayManager::getSingleton().getOverlayElement("TimeLeft");
		mElementAltitude = Ogre::OverlayManager::getSingleton().getOverlayElement("Altitude");
		mElementDistance = Ogre::OverlayManager::getSingleton().getOverlayElement("Distance");
		mElementSpeed = Ogre::OverlayManager::getSingleton().getOverlayElement("Speed");

		mOverlayEndGame = Ogre::OverlayManager::getSingleton().getByName("EndGame");
		mElementEndScore = Ogre::OverlayManager::getSingleton().getOverlayElement("EndScore");
		mElementEndDistance = Ogre::OverlayManager::getSingleton().getOverlayElement("EndDistance");
	}

	RubyHud::~RubyHud()
	{
		mOverlayHud->hide();
		mElementScore->hide();
		mElementTimeLeft->hide();
		mElementAltitude->hide();
		mElementDistance->hide();
		mElementSpeed->hide();

		mOverlayEndGame->hide();
		mElementEndScore->hide();
		mElementEndDistance->hide();
	}

	void RubyHud::show(bool show)
	{
		if(show)
		{
			mOverlayHud->show();
			mElementScore->show();
			mElementTimeLeft->show();
			mElementAltitude->show();
			mElementDistance->show();
			mElementSpeed->show();
		}
		else
		{
			mOverlayHud->hide();
			mElementScore->hide();
			mElementTimeLeft->hide();
			mElementAltitude->hide();
			mElementDistance->hide();
			mElementSpeed->hide();
		}
	}

	void RubyHud::update(const Ogre::Vector3& rubySpeed, double rubyAltitude, double elapsedTime, double score)
	{
		mSpeed = rubySpeed.z;
		mAltitude = rubyAltitude;
		mDistance += (mSpeed * elapsedTime);
		mScore = score;
		mTimer += elapsedTime;

		std::ostringstream oss;
		oss << "Score : " << (int)score << " pts";
		mElementScore->setCaption(oss.str());
		oss.str("");
		oss << "Time : " << Properties::GAME_LENGTH - (int)mTimer;
		mElementTimeLeft->setCaption(oss.str());
		oss.str("");
		oss << "Altitude : " << (int)mAltitude << " m";
		mElementAltitude->setCaption(oss.str());
		oss.str("");
		oss << "Speed : " << (int)(mSpeed * 3.6) << " km/h";
		mElementSpeed->setCaption(oss.str());
		oss.str("");
		oss << "Distance : " << (int)mDistance << " m";
		mElementDistance->setCaption(oss.str());
	}

	void RubyHud::showGameOver(double elapsedTime)
	{	
		mGameOverTimer += elapsedTime;
		
		if(mGameOverTimer > 1)
		{
			std::ostringstream oss;
			oss << "Score : " << (int)mScore << " points";
			mElementEndScore->setCaption(oss.str());
			oss.str("");
			oss << "Distance : " << (int)mDistance << " m";
			mElementEndDistance->setCaption(oss.str());

			mOverlayEndGame->show();
			mElementEndScore->show();
			mElementEndDistance->show();
			mGameOverTimer = 0;
		}
	}
}