#include "Stopper.h"
#include "StopperBody.h"
#include "GameEngine.h"
#include "Properties.h"
#include "RubyGameRenderer.h"

namespace TR
{
	Stopper::Stopper(GameEngine* gameEngine):
		EnvironmentObject(gameEngine),
		mActivated(false)
	{
		mBody = new StopperBody(this);
		mSoundChannel = 1;
	}

	Stopper::~Stopper()
	{
		mGameEngine->getSoundManager()->StopSound(&mSoundChannel);
	}

	void Stopper::onCollide(Object* obj)
	{
		if(!mActivated)
		{
			// Affect target.
			obj->setLinearSpeed(Ogre::Vector3(0,0,0));
			obj->accelerate(Ogre::Vector3(0,-20,0));

			// Change camera.
			RubyGameRenderer* gameRenderer = static_cast<RubyGameRenderer*>(mGameEngine->getGameRenderer());
			gameRenderer->setCameraObjectOptions(mBody, RubyGameRenderer::AT_FIXED, 60, Ogre::Vector3(13, 20, -6));
			gameRenderer->setCameraMode(RubyGameRenderer::CM_OBJECT_CAMERA);

			// Play sound.
			int sound = mGameEngine->getSoundManager()->CreateSound(Ogre::String("stop.ogg"), SOUND_TYPE_3D_SOUND);
			mGameEngine->getSoundManager()->PlaySound(sound, mBody->getSceneNode(), &mSoundChannel);
			mGameEngine->getSoundManager()->Set3DMinMaxDistance(mSoundChannel, Properties::SOUND_3D_MIN, Properties::SOUND_3D_MAX);
			
			mActivated = true;
		}
	}
}