#include "TerrainScroller.h"

using namespace Ogre;

namespace TR
{
	TerrainScroller::TerrainScroller(const Vector2& tileDimensions,
		const Vector2& nbTiles,
		const Vector3& center,
		const Vector3& depthDirection,
		const Vector3& widthDirection,
		const Vector2& speed)
	{
		mTileDimensions = tileDimensions;
		mNbTiles = nbTiles;
		mCenter = center;
		mDepthDirection = depthDirection;
		mWidthDirection = widthDirection;
		mSpeed = speed;
		mMaximumY = Math::NEG_INFINITY;
		mSceneNode = NULL;
		srand(time(0));
	}

	TerrainScroller::~TerrainScroller(){}

	void TerrainScroller::initialize(SceneManager* sm, const String& name, const std::vector<std::pair<String,double>>& mesh)
	{
		Entity* entity;
		SceneNode* node = sm->getRootSceneNode()->createChildSceneNode();
		mSceneNode = node;
		mSceneNode->setPosition(mCenter);
		
		std::ostringstream oss;
		
		mDepthDirection.normalise();
		Vector3 widthLimit = mTileDimensions.x * mNbTiles.x * 0.5 * mWidthDirection;
		Vector3 depthLimit = mTileDimensions.y * mNbTiles.y * 0.5 * mDepthDirection;
		
		size_t nb = mesh.size();

		for(size_t i=1; i<=mNbTiles.x; i++)
		{
			for(size_t j=1; j<=mNbTiles.y; j++)
			{
				oss.str("");
				oss << name << i << "_" << j;

				// Selects a mesh from the list with a given weight
				double randomMesh = Math::UnitRandom();
				double sum = 0;
				bool stop = false;
				for(size_t k=0; k<nb && stop==false; k++)
				{
					sum += mesh[k].second;
					if(sum > randomMesh || k == nb-1){
						entity = sm->createEntity(oss.str(), mesh[k].first);
						stop = true;
					}
				}

				SceneNode* subNode = node->createChildSceneNode();
				subNode->attachObject(entity);
				subNode->setPosition(widthLimit - mTileDimensions.x * (i-0.5) * mWidthDirection
					+ depthLimit - mTileDimensions.y * (j-0.5) * mDepthDirection);

				double tmp = entity->getBoundingBox().getMaximum().y;
				if(mMaximumY < tmp)
					mMaximumY = tmp;
			}
		}
	}

	void TerrainScroller::update(double elapsedTime)
	{
		Vector2 distance = mSpeed * elapsedTime;
		Vector3 widthLimit, depthLimit;

		if(mSpeed.x > 0)
			widthLimit = mTileDimensions.x * mNbTiles.x * 0.5 * mWidthDirection;
		else
			widthLimit = mTileDimensions.x * mNbTiles.x * (-0.5) * mWidthDirection;

		if(mSpeed.y > 0)
			depthLimit = mTileDimensions.y * mNbTiles.y * 0.5 * mDepthDirection;
		else
			depthLimit = mTileDimensions.y * mNbTiles.y * (-0.5) * mDepthDirection;

		SceneNode* subNode = NULL;

		Node::ChildNodeIterator iterator = mSceneNode->getChildIterator();
		while(iterator.hasMoreElements())
		{
			subNode = static_cast<SceneNode*>(iterator.getNext());
			subNode->translate(mDepthDirection * distance.y + mWidthDirection * distance.x);

			if(depthLimit.squaredLength() < (subNode->getPosition()*mDepthDirection).squaredLength())
				subNode->setPosition(subNode->getPosition()- 2 * depthLimit);

			else if(widthLimit.squaredLength() < (subNode->getPosition()*mWidthDirection).squaredLength())
				subNode->setPosition(subNode->getPosition()- 2 * widthLimit);
				
		}
	}

	void TerrainScroller::showBoundingBox(bool bShow)
	{
		SceneNode* subNode = NULL;

		Node::ChildNodeIterator iterator = mSceneNode->getChildIterator();
		while(iterator.hasMoreElements())
		{
			subNode = static_cast<SceneNode*>(iterator.getNext());
			subNode->showBoundingBox(bShow);
		}
	}

}
